@@ -1,4 +1,5 @@
 import inspect
+from collections import UserList
 from contextlib import contextmanager
 from contextvars import ContextVar, Token
 from functools import lru_cache
@@ -18,7 +19,6 @@ from typing import (
     Union,
 )
 
-
 from .entities.dispatcher import BaseDispatcher
 
 if TYPE_CHECKING:
@@ -41,7 +41,7 @@ async def run_always_await_safely(callable, *args, **kwargs):
 
 
 T = TypeVar("T")
-D = TypeVar("D")
+D = TypeVar("D", type(None), Any)
 
 
 class Ctx(Generic[T]):
@@ -50,7 +50,7 @@ class Ctx(Generic[T]):
     def __init__(self, name: str) -> None:
         self.current_ctx = ContextVar(name)
 
-    def get(self, default: Union[T, D] = None) -> Union[T, D]:
+    def get(self, default: Optional[Union[T, D]] = None) -> Union[T, D]:
         return self.current_ctx.get(default)
 
     def set(self, value: T):
@@ -71,6 +71,20 @@ def printer(value: Any):
     return value
 
 
+class DebugList(UserList):
+    def extend(self, item) -> None:
+        print(item)
+        return super().extend(item)
+
+    def append(self, item) -> None:
+        print(item)
+        return super().append(item)
+
+    def insert(self, i: int, item) -> None:
+        print(i, item)
+        return super().insert(i, item)
+
+
 def group_dict(iterable: Iterable, key_callable: Callable[[Any], Any]):
     temp = {}
     for i in iterable:
